#!/usr/bin/php
<?php
include "/usr/local/opmon/etc/config.php";

$mod = "EXTERNAL";
set_logged_user("opmonadmin");

function main() {

    $options = getopt("s:i:d:h");

    if (isset($options['h']))
        usage();

    if (!isset($options['s']) && !isset($options['i']))
        usage();
    $myscat = false;
    if (isset($options['s'])) {
        $myscat = get_scat_by_name($options['s']);
    } else if (isset($options['i'])) {
        if (! is_numeric($options['i'])) {
            print "Invalid ID".PHP_EOL;
            exit(UNKNOWN);
        }
        $myscat = get_scat_by_id($options['i']);
    }

    if ($myscat == false) {
        print "Unable to locate service catalog";
        exit(UNKNOWN);
    }

    $code = $myscat->getStatusForMonitoring();

    if (count($myscat->levels) == 0) {
        print "Catálogo sem elementos.\n";
        update_scat_status($options['i'], OK);
        exit(OK);
    }

    if ($code == 0) {
        print "Servico do catalogo em funcionamento normal.\n";
        update_scat_status($options['i'], $code);
        exit(OK);
    }

    $stateDesc = array(
        0 => "OK",
        1 => "DOWNTIME",
        2 => "ACKNOWLEDGED",
        3 => "FLAPPING",
        4 => "WARNING",
        5 => "UNKNOWN",
        6 => "CRITICAL",
        7 => "UNREACHABLE",
        8 => "DOWN"
    );

    $plugin_output = "%d elementos do servico com problema: %s\n";
    $prbl_elems = 0;
    $problems = array();
    foreach($myscat->levels as $level => $elements) {
        foreach($elements as $e) {
            if ($e->object_type === SERVICECATALOG) { // scat inside scat
                $s = get_scat_by_id($e->host_id);
                if (! $s) {
                    $problems[] = "Unable to locate service catalog by id $e->host_id";
                }
                $state = $s->get_status();
                $desc = "{$s->name}";
            }

            if ($e->object_type === SERVICE) {
                $s = get_service_by_id($e->host_id, $e->service_id);
                if (! $s) {
                    $problems[] = "Unable to locate service({$e->host_id},{$e->service_id})";
                }
                $status = $s->get_status();
                $state = $status->display_status;
                $desc = "{$status->host_name}({$status->description})";
            }

            if ($e->object_type === HOST) {
                $h = get_host_by_id($e->host_id);
                if (! $h) {
                    $problems[] = "Unable to locate host({$e->host_id})";
                }
                $status = $h->get_status();
                $state = $status->display_status;
                $desc = "{$status->host_name}";
            }

            if ($state >= 1) {
                $prbl_elems++;
                $desc .= " ".$stateDesc[$state];
                $problems[] = $desc;
            }
        }
    }

    if ($code >= 1) {
        $exit_code = 0;
    }

    if ($code >= 2) {
        $exit_code = 1;
    }

    if ($code == 5) {
        $exit_code = 3;
    }

    if ($code >= 6) {
        $exit_code = 2;
    }

    print sprintf($plugin_output, $prbl_elems, implode(",",$problems));
    update_scat_status($options['i'], $code);
    exit($exit_code);
}

function usage() {
    print "Usage: ./check_scat_status.php [-s <scat name>] [-i <scat id>] [-d [1|2]]\n";
    print "\t -d 1 - Force scheduled downtimes as OK \n";
    print "\t -d 2 - Force scheduled downtimes as WARNING \n";
    exit(UNKNOWN);
}

main();
