#!/usr/bin/sudo /bin/bash
#
#set -x

clean_mount_point () {
	umount ${MOUNTPOINT} > /dev/null 2>/dev/null
	rmdir ${MOUNTPOINT} > /dev/null 2>/dev/null

}

if [ ! $# == 5 ]; then
    echo "Usage: $0 warning_percent_level critical_percent_level smb_user smb_password smb_url"
    exit 4
fi

W=$1
C=$2
SMBUSER=$3
SMBPASS=$4
SMBURL=$5

MOUNTPOINTTMP=`echo "${SMBURL}" | tr / _`
MOUNTPOINT='/usr/local/opmon/libexec/opservices/tmp/mounts/'${MOUNTPOINTTMP}
mkdir -p ${MOUNTPOINT}

ERROR=`/bin/mount -t cifs -o username=${SMBUSER},password=${SMBPASS} ${SMBURL} ${MOUNTPOINT} >/dev/null 2>/dev/null`
if [ $? -gt 0 ]; then
	echo "Error mounting ${SMBURL} $ERROR"
	clean_mount_point ${MOUNTPOINT}
	exit 3
fi

LINE=`df -P | grep ${MOUNTPOINTTMP}`
TOTAL_BLOCKS=`echo ${LINE} | cut -d " " -f 2 | grep -o -e [0-9]\*`
USED_BLOCKS=`echo ${LINE} | cut -d " " -f 3 | grep -o -e [0-9]\*`
AVAIL_BLOCKS=`echo ${LINE} | cut -d " " -f 4 | grep -o -e [0-9]\*`
PERCENT=`echo ${LINE} | cut -d " " -f 5 | grep -o -e [0-9]\*`

BLOCK_SIZE=1024
FREEGB=`echo $AVAIL_BLOCKS*$BLOCK_SIZE/1024/1024/1024 | bc`
TOTALGB=`echo $TOTAL_BLOCKS*$BLOCK_SIZE/1024/1024/1024 | bc`
USED_BLOCKS=`echo $USED_BLOCKS*$BLOCK_SIZE/1024/1024/1024 | bc`

PERFDATA="free=${FREEGB}GB;;;0;${TOTALGB} total=${TOTALGB}GB;;;0;${TOTALGB} used=${USED_BLOCKS}GB;;;0;${TOTALGB}"

clean_mount_point ${MOUNTPOINT}

state=3
msg="UNKNOWN"

if [ $PERCENT -gt $C ]; then
	state=2
	msg="CRITICAL"	
elif [ $PERCENT -gt $W ]; then
        state=1
        msg="WARNING"
else
        state=0
        msg="OK"
fi

echo "$msg: $FREEGB GB free of $TOTALGB GB ($PERCENT %) | ${PERFDATA}"
exit $state

