#!/usr/bin/php
<?php

require "../../share/ws/nusoap/lib/nusoap.php";

define("WSDL", "https://localhost/opmon/ws/");

$client = null;

function main()
{
    global $client;

    $options = getOptions();

    $servicename = $options['servicename'] ?: false;

    $state = ($options['servicestate'] ?: $options['hoststate']);

    $typeOfFeedback = 'acknowledgement';

    if (array_key_exists('c', $options)) {
        $typeOfFeedback = 'comment';
    }

    $client  = getSoapConnection($options['username'], $options['password']);
    $comment = generateComment($options['hostname'], $state);

    $id = getOpenedTicketId($options['hostname'], $options['servicename']);
    if (!is_null($id) && ($state == 'OK' || $state == 'UP')) {
        addTicketHistory($id, $comment);
        return closeTicket($id);
    }

    if (!is_null($id)) {
        return addTicketHistory($id, $comment);
    }

    openNewTicket($options['hostname'], $comment);
    doFeedback($typeOfFeedback, $options['hostname'], $comment, $servicename);
}

function openNewTicket($hostname, $comment, $servicename = false)
{
    /**
     * Implement function to open new ticket.
     */
    return $id;
}

function closeTicket($tickedid)
{
    /**
     * Implement function to close ticket.
     */
}

function generateComment($host, $state, $servicename = false)
{
    /**
     * Implement function to generate the comment to insert in host or servicename.
     */
    return $comment;
}

function addTicketHistory($tickedId, $comment)
{
    /**
     * Implement function to add note on ticket already opened.
     */
}

function getOpenedTicketId($hostname, $servicename = false)
{
    /**
     * Implement function to verify if already ticket for host or servicename*
     */
    return $ticketId;
}

function doFeedback($typeOfFeedback, $hostname, $comment, $servicename = false)
{
    $fn = ($typeOfFeedback == "comment") ? 'addCommentToOpMon' : 'addAcknowledgementToOpMon';
    return $fn($hostname, $comment, $servicename);
}

function addCommentToOpMon($hostname, $comment, $servicename = false)
{
    global $client;
    return $client->call('soap_add_comment', array($hostname, $comment, $servicename));
}

function addAcknowledgementToOpMon($hostname, $comment, $servicename = false)
{
    global $client;
    return $client->call('soap_add_acknowledgement', array($hostname, $comment, $servicename));
}

function getSoapConnection($username, $password)
{
    $client = new nusoap_client(WSDL);
    $client->soap_defencoding = 'UTF-8';
    $client->decode_utf8 = false;

    $client->setCredentials($username, $password);
    return $client;
}

function getOptions()
{
    $longOpts = array(
    	"hostname:",
    	"hoststate:",
    	"hostoutput:",
    	"servicename:",
    	"servicestate:",
    	"serviceoutput:",
    	"username:",
    	"password:"
    );

    $options =  getopt('c::h', $longOpts);

    if (isset($options['h'])) {
    	help();
    	exit(2);
    }
    return $options;
}

function help()
{
    print "\n";
    print "How it works:\n\n";
    print "generic-integration-script.php makes use of Nagios exported macros to determine\n";
    print "a lot of its needs. Due to this approach, generic-integration-script only works\n";
    print "inside a running Nagios environment, if you want to test it outside\n";
    print "Nagios' environment, you may export those macros as shell variables\n";
    print "before running it, for example: \$ export HOSTNAME='hostX'\n\n";
    print "Among the needed macros, generic-integration-script.php uses:\n";
    print "\tGeneral macros:\n";
    print "\t\tHOSTNAME\n";
    print "\n\tHost specific macros:\n";
    print "\t\tHOSTSTATE\n";
    print "\t\tHOSTOUTPUT\n";
    print "\n\tService specific macros:\n";
    print "\t\tSERVICEDESC\n";
    print "\t\tSERVICESTATE\n";
    print "\t\tSERVICEHOSTOUTPUT\n";
    print "\n";
    print 'generic-integration-script.php --hostname="$HOSTNAME$" --hoststate="$HOSTSTATE$" --hostoutput="$HOSTOUTPUT$" --servicename"$SERVICEDESC$" --servicestate="$SERVICESTATE$" --serviceoutput="$SERVICEOUTPUT$" --username="useropmon" --password="password"';
    print "\n";
    print "By default, the command insert an acknoledgement in OpMon, case needs insert a\n";
    print "comment, is necessary use the flag -c\n";
}

main();
