#!/usr/bin/php
<?php
include "/usr/local/opmon/etc/config.php";
$mod = "EXTERNAL";
set_logged_user("opmonadmin");

define("_VERSION", "1.0");

$valid_periods = array(
    "lasthour",
    "today",
    "last24hours",
    "thisweek",
    "last7days",
    "thismonth",
    "last31days",
    "thisyear",
    "thishour"
);

$options = array();

function main() {

    global $valid_periods;
    global $options;
    $options = getopt("CjSsvVhmt:o:w:c:p:T:");

    if (isset($options['V'])) {
        usage(true);
        return UNKNOWN;
    }

    if (isset($options['h'])) {
        usage();
        return UNKNOWN;
    }

    if (! isset($options['t']) || ! isset($options['o'])) {
        usage();
        return UNKNOWN;
    }

    $jsonreturn = false;
    if (isset($options['j'])) {
        $jsonreturn = true;
    }

    $tcname = "24x7";
    if (isset($options['T'])) {
        $tcname = $options['T'];
    }

    $tc = get_timecut_by_name($tcname);
    if ($tc === false) {
        print "Unable to locate timecut $tcname\n";
        return UNKNOWN;
    }

    $host_name = '';
    $service_description = '';
    $objects = get_objects(
        $options['t'],
        $options['o'],
        $host_name,
        $service_description
    );

    if (empty($objects)) {
        print "Unable to locate object {$options['o']}. Please verify type(-t) and description(-o)\n";
        return UNKNOWN;
    }

    $period = "thismonth";
    if (isset($options['p'])) {
        $period = $options['p'];
    }

    if (array_search($period,$valid_periods) === false) {
        print "Unknown/invalid period {$period}\n";
        return UNKNOWN;
    }

    if (isset($options['C'])) {
        $stats = getStateChanges($objects, $period, isset($options['S']));
    }

    $time_ok = 0;
    $time_warning = 0;
    $time_critical = 0;
    $time_unknown = 0;

    $ok = 0;
    $warning = 0;
    $critical = 0;
    $unknown = 0;
    $max_ok = false;

    foreach($objects as $obj) {
        if (get_class($obj) !== 'Scat') {
            $obj->get_avail_instance();

            if (isset($options['s'])) {
                $obj->avail->setSchedAsOk();
                $obj->avail->setSlaBrAsOk();
            }

            if (isset($options['S'])) {
                $obj->avail->setSoftState();
            }

            $obj->avail->setPeriod($period);
            $obj->avail->setTimeCutId($tc->id);
            $obj->avail->calcule();

            $time_ok += $obj->avail->timeOk;
            $time_warning += $obj->avail->timeWarning;
            $time_unknown += $obj->avail->timeUnknown;
            $time_critical += $obj->avail->timeCritical;

            $ok += $obj->avail->timePctOk;
            $warning += $obj->avail->timePctWarning;
            $critical += $obj->avail->timePctCritical;
            $unknown += $obj->avail->timePctUnknown;

            if ($obj->avail->maxTimePctOk !== false) {
                $max_ok += $obj->avail->maxTimePctOk;
            }

        } else {

            if (isset($options['s'])) {
                $obj->set_sched_as_ok();
            }

            if (isset($options['S'])) {
                $obj->set_soft_states();
            }

            $obj->set_period($period);
            $obj->set_timecut_id($tc->id);
            $obj->calcule();

            $ok += $obj->time_pct_ok;
            $critical += $obj->time_pct_critical;

            $time_ok += $obj->time_ok;
            $time_critical += $obj->time_critical;

            if ($obj->max_time_pct_ok !== false) {
                $max_ok += $obj->max_time_pct_ok;
            }
        }

        unset($obj->avail);
    }

    $count = count($objects);
    if ($count == 0) {
        $count = 1;
    }

    $time_ok = sprintf("%.1f", ($time_ok/$count) / 60 );
    $time_warning = sprintf("%.1f", ($time_warning/$count) / 60 );
    $time_critical = sprintf("%.1f", ($time_critical/$count) / 60);
    $time_unknown = sprintf("%.1f", ($time_unknown/$count) / 60 );

    $ok = sprintf("%.3f",$ok / $count);
    $warning = sprintf("%.3f",$warning / $count);
    $critical = sprintf("%.3f",$critical / $count);
    $unknown = sprintf("%.3f",$unknown / $count);
    if ($max_ok !== false) {
        $max_ok = sprintf("%.3f",$max_ok / $count);
    }

    $exit_code = OK;
    if ($ok < $options['c']) {
        $exit_code = CRITICAL;
    } else if ($ok < $options['w']) {
        $exit_code = WARNING;
    }

    if ($jsonreturn === true) {
        $out = array('custom' => 'custom value '.date('H:i:s'),
            'host_name' => "{$host_name}",
            'service_description' => "{$service_description}",
            'display_status' => $exit_code);
        if ($exit_code == CRITICAL) {
            $out['display_status'] = 7;
        }
        if ($exit_code == WARNING) {
            $out['display_status'] = 4;
        }
    }

    // OUTPUT FORMAT
    $metrics = array();
    $data = array();
    $aux = array();
    $aux['label'] = 'label 0';
    $plugin_ouput = '';
    $plugin_ouput.=$options['o'].": ";
    $total = array_sum(array_values($stats));
    if ($options['t'] == 'IC' || $options['t'] == 'ICG') {
        $plugin_ouput.="UP: $ok% DOWN: $critical% UNREACHABLE: $unknown% |";
        $plugin_ouput.="up=$ok;{$options['w']};{$options['c']};0;100 ";
        $plugin_ouput.="down=$critical;;;0;100 ";
        $plugin_ouput.="unreachable=$unknown;;;0;100 ";
        if (isset($options['C'])) {
            $plugin_ouput.="total_up={$stats['ok']};;;0;{$total} ";
            $plugin_ouput.="total_down={$stats['down']};;;0;{$total} ";
            $plugin_ouput.="total_flapping={$stats['flapping']};;;0;{$total} ";
            $plugin_ouput.="total_downtime={$stats['downtime']};;;0;{$total} ";
            $plugin_ouput.="total_acknowledged={$stats['acknowledged']};;;0;{$total} ";
            $plugin_ouput.="total_unreachable={$stats['unreachable']};;;0;{$total}";
        }

        if ($jsonreturn === true) {
            $metrics['1001'] = array('value' => $ok,
                'warning' => "{$options['w']}",
                'critical' => "{$options['c']}",
                'metric' => 'up',
                'max' => '100',
                'min' => '0',
                'value_max' => '100',
                'value_min' => '0',
                'unit' => '%');
            $aux['up'] = $ok;
            $out['up'] = $ok;
            $metrics['1002'] = array('value' => $critical,
                'warning' => "{$options['w']}",
                'critical' => "{$options['c']}",
                'metric' => 'down',
                'max' => '100',
                'min' => '0',
                'value_max' => '100',
                'value_min' => '0',
                'unit' => '%');
            $aux['down'] = $critical;
            $out['down'] = $critical;
            $metrics['1003'] = array('value' => $unknown,
                'warning' => "{$options['w']}",
                'critical' => "{$options['c']}",
                'metric' => 'unreachable',
                'max' => '100',
                'min' => '0',
                'value_max' => '100',
                'value_min' => '0',
                'unit' => '%');
            $aux['unreachable'] = $unknown;
            $out['unreachable'] = $unknown;

            if (isset($options['C'])) {
                $index = 0;
                foreach ($stats as $key => $value) {
                    if (in_array($key, array('critical', 'warning', 'unknown'))) {
                        continue;
                    }
                    $metricName = ($key === 'ok') ? 'total_up' : 'total_'.$key;
                    $metrics[1010 + $index] = array('value' => $value,
                        'warning' => "",
                        'critical' => "",
                        'metric' => $metricName,
                        'max' => $total,
                        'min' => 0,
                        'value_max' => $total,
                        'value_min' => 0,
                        'unit' => '');
                    $aux[$metricName] = $value;
                    $out[$metricName] = $value;
                    $index++;
                }
            }
        }

        if (isset($options['m'])) {
            $plugin_ouput.=" minutes_up=$time_ok;;;;";
            $plugin_ouput.=" minutes_down=$time_critical;;;;";
            $plugin_ouput.=" minutes_unreachable=$time_unknown;;;;";
            if ($jsonreturn === true) {
                $metrics['1004'] = array('value' => $time_ok,
                    'warning' => '',
                    'critical' => '',
                    'metric' => 'minutes_up',
                    'max' => '',
                    'min' => '',
                    'value_max' => '',
                    'value_min' => '',
                    'unit' => 'm');
                $aux['minutes_up'] = $time_ok;
                $out['minutes_up'] = $time_ok;
                $metrics['1005'] = array('value' => $time_critical,
                    'warning' => '',
                    'critical' => '',
                    'metric' => 'minutes_down',
                    'max' => '',
                    'min' => '',
                    'value_max' => '',
                    'value_min' => '',
                    'unit' => 'm');
                $aux['minutes_down'] = $time_critical;
                $out['minutes_down'] = $time_critical;
                $metrics['1006'] = array('value' => $time_unknown,
                    'warning' => '',
                    'critical' => '',
                    'metric' => 'minutes_unreachable',
                    'max' => '','min' => '',
                    'value_max' => '',
                    'value_min' => '',
                    'unit' => '');
                $aux['minutes_unreachable'] = $time_unknown;
                $out['minutes_unreachable'] = $time_unknown;
            }
        }

        if ($max_ok !== false) {
            $plugin_ouput.=" max_up=$max_ok;;;0;100";
        }

    } else if ($options['t'] == 'AIC' || $options['t'] == 'AICG') {
        $plugin_ouput.="OK: $ok% WARNING: $warning% CRITICAL: $critical% UNKNOWN: $unknown% |";
        $plugin_ouput.="ok=$ok;{$options['w']};{$options['c']};0;100 ";
        $plugin_ouput.="warning=$warning;;;0;100 ";
        $plugin_ouput.="critical=$critical;;;0;100  ";
        $plugin_ouput.="unknown=$unknown;;;0;100 ";
        if (isset($options['C'])) {
            $plugin_ouput.="total_ok={$stats['ok']};;;0;{$total} ";
            $plugin_ouput.="total_warning={$stats['warning']};;;0;{$total} ";
            $plugin_ouput.="total_critical={$stats['critical']};;;0;{$total} ";
            $plugin_ouput.="total_flapping={$stats['flapping']};;;0;{$total} ";
            $plugin_ouput.="total_downtime={$stats['downtime']};;;0;{$total} ";
            $plugin_ouput.="total_acknowledged={$stats['acknowledged']};;;0;{$total} ";
            $plugin_ouput.="total_unknown={$stats['unknown']};;;0;{$total}";
        }

        if ($jsonreturn === true) {
            $metrics['1001'] = array('value' => $ok,
                'warning' => "{$options['w']}",
                'critical' => "{$options['c']}",
                'metric' => 'ok',
                'max' => '100',
                'min' => '0',
                'value_max' => '100',
                'value_min' => '0',
                'unit' => '%');
            $aux['ok'] = $ok;
            $out['ok'] = $ok;

            $metrics['1002'] = array('value' => $critical,
                'warning' => "{$options['w']}",
                'critical' => "{$options['c']}",
                'metric' => 'critical',
                'max' => '100',
                'min' => '0',
                'value_max' => '100',
                'value_min' => '0',
                'unit' => '%');
            $aux['critical'] = $critical;
            $out['critical'] = $critical;

            $metrics['1003'] = array('value' => $warning,
                'warning' => "{$options['w']}",
                'critical' => "{$options['c']}",
                'metric' => 'warning',
                'max' => '100',
                'min' => '0',
                'value_max' => '100',
                'value_min' => '0',
                'unit' => '%');
            $aux['warning'] = $warning;
            $out['warning'] = $warning;

            $metrics['1004'] = array('value' => $unknown,
                'warning' => "{$options['w']}",
                'critical' => "{$options['c']}",
                'metric' => 'unknown',
                'max' => '100',
                'min' => '0',
                'value_max' => '100',
                'value_min' => '0',
                'unit' => '%');
            $aux['unknown'] = $unknown;
            $out['unknown'] = $unknown;

            if (isset($options['C'])) {
                $index = 0;
                foreach ($stats as $key => $value) {
                    if (in_array($key, array('down', 'unreachable'))) {
                        continue;
                    }
                    $metricName = 'total_'.$key;
                    $metrics[1010 + $index] = array(
                        'value' => $value,
                        'warning' => "",
                        'critical' => "",
                        'metric' => $metricName,
                        'max' => $total,
                        'min' => 0,
                        'value_max' => $total,
                        'value_min' => 0,
                        'unit' => ''
                    );
                    $aux[$metricName] = $value;
                    $out[$metricName] = $value;
                    $index++;
                }
            }
        }

        if (isset($options['m'])) {
            $plugin_ouput.=" minutes_ok=$time_ok;;;;";
            $plugin_ouput.=" minutes_warning=$time_warning;;;;";
            $plugin_ouput.=" minutes_critical=$time_critical;;;;";
            $plugin_ouput.=" minutes_unknown=$time_unknown;;;;";
        }

        if ($jsonreturn === true) {
            $metrics['1005'] = array('value' => $time_ok,
                'warning' => '',
                'critical' => '',
                'metric' => 'minutes_ok',
                'max' => '',
                'min' => '',
                'value_max' => '',
                'value_min' => '',
                'unit' => 'm');
            $aux['minutes_ok'] = $time_ok;
            $out['minutes_ok'] = $time_ok;

            $metrics['1006'] = array('value' => $time_critical,
                'warning' => '',
                'critical' => '',
                'metric' => 'minutes_critical',
                'max' => '',
                'min' => '',
                'value_max' => '',
                'value_min' => '',
                'unit' => 'm');
            $aux['minutes_critical'] = $time_critical;
            $out['minutes_critical'] = $time_critical;

            $metrics['1007'] = array('value' => $time_warning,
                'warning' => '',
                'critical' => '',
                'metric' => 'minutes_warning',
                'max' => '','min' => '',
                'value_max' => '',
                'value_min' => '',
                'unit' => '');
            $aux['minutes_warning'] = $time_warning;
            $out['minutes_warning'] = $time_warning;

            $metrics['1008'] = array('value' => $time_unknown,
                'warning' => '',
                'critical' => '',
                'metric' => 'minutes_unknown',
                'max' => '','min' => '',
                'value_max' => '',
                'value_min' => '',
                'unit' => '');
            $aux['minutes_unknown'] = $time_unknown;
            $out['minutes_unknown'] = $time_unknown;
        }
        if ($max_ok !== false) {
            $plugin_ouput.=" max_ok=$max_ok;;;0;100";
        }

    } else {
        $plugin_ouput.="OK: $ok% CRITICAL: $critical% |";
        $plugin_ouput.="ok=$ok;{$options['w']};{$options['c']};0;100 ";
        $plugin_ouput.="critical=$critical;;;0;100 ";
        if (isset($options['C'])) {
            $plugin_ouput.="total_ok={$stats['ok']};;;0;{$total} ";
            $plugin_ouput.="total_critical={$stats['critical']};;;0;{$total}";
        }

        if (isset($options['m'])) {
            $plugin_ouput.=" minutes_ok=$time_ok;;;;";
            $plugin_ouput.=" minutes_critical=$time_critical;;;;";
        }

        if ($max_ok !== false) {
            $plugin_ouput.=" max_ok=$max_ok;;;0;100";
        }

    }

    if ($jsonreturn === true) {
        $out['metrics'] = $metrics;
        $out['data'][] = $aux;
        print json_encode($out);
    }
    else {
        print $plugin_ouput."\n";
    }
    return $exit_code;
}

function getStateChanges($objects, $period, $withSoft)
{
    global $options;
    $conn = Livestatus\Connection\ConnectionProvider::factory();
    $query = $conn->from('log');
    //$query->select('state state_type class plugin_output');

    $stats = array(
        'stats_1' => 'ok',
        'stats_2' => 'downtime',
        'stats_3' => 'acknowledged',
        'stats_4' => 'flapping',
        'stats_5' => 'warning',
        'stats_6' => 'unknown',
        'stats_7' => 'critical',
        'stats_8' => 'unreachable',
        'stats_9' => 'down'
    );

    for ($i = 0; $i <= 8; $i++) {
        $query->statsFor(
            $conn->createStats(
                array('state', $conn->createOperator('='), $i)
            )
        );
    }

    list($start, $end) = get_time_stamps($period);
    $query->filterBy(
        $conn->createFilter(
            array(
                array('time', $conn->createOperator('>='), $start),
                array('time', $conn->createOperator('<='), $end),
                array('class', $conn->createOperator('='), 1),
            ),
            $conn->createOperator('And')
        )
    );

    if (! $withSoft) {
        $query->filterBy(
            $conn->createFilter(
                array('state_type', $conn->createOperator('='), 'HARD')
            )
        );
    }

    $filters = array();
    foreach ($objects as $object) {
        if (get_class($object) === "Service") {
            $type = "Service";
            $filters[] = $conn->createFilter(
                array(
                    array(
                        'host_name',
                        $conn->createOperator('='),
                        $object->get_host_name()
                    ),
                    array(
                        'service_description',
                        $conn->createOperator('='),
                        $object->get_service_description()
                    )
                ),
                $conn->createOperator("And")
            );
        }

        if (get_class($object) === "Host") {
            $type = "Host";
            $filters[] = array(
                'host_name',
                $conn->createOperator('='),
                $object->get_host_name()
            );
        }
    }

    if (count($filters) === 1 && $type === "Host") {
        $query->filterBy(
            $conn->createFilter(
                $filters[0]
            )
        );
    }

    if (count($filters) === 1 && $type === "Service") {
        $query->filterBy($filters[0]);
    }

    if (count($filters) > 1) {
        $query->filterBy(
            $conn->createFilter(
                $filters,
                $conn->createOperator('Or')
            )
        );
    }

    $result = $conn->execute()->fetchAll();

    if (isset($options['v'])) {
        echo $query;
        print_r($result);
    }

    $data = array();
    foreach ($result[0] as $key => $value) {
        $data[$stats[$key]] = $value;
    }

    return $data;
}

function get_objects($type, $object, &$host_name, &$service_description)
{
    $objects = array();
    switch($type) {

        // host
    case 'IC':
        $host_name = $object;
        $service_description = '';
        $host = get_host_by_name($object);

        if ($host === false)
            break;

        $objects[] = $host;
        break;

        // service
    case 'AIC':
        list($host_name, $service_description) = explode(":",$object);

        $host = get_host_by_name($host_name);
        if ($host !== false) {

            $services = $host->get_services();
            foreach($services as $svc) {
                if ($svc->get_service_description() == $service_description) {
                    $objects[] = $svc;
                }
            }
        }
        break;

        // hostgroup
    case "ICG":
        $host_name = $object;
        $service_description = '';
        $hgs = get_hostgroups();
        if (is_array($hgs)) {
            foreach($hgs as $hg) {
                if ($hg->hostgroup_name == $object) {
                    $host = get_host($hg->host_id);
                    if ($host !== false) {
                        $objects[] = $host;
                    }
                }
            }
        }
        break;

        // servicegroup
    case "AICG":
        $host_name = $object;
        $service_description = '';
        $sgs = get_servicegroups();
        if (is_array($sgs)) {
            foreach($sgs as $sg) {
                if ($sg->servicegroup_name == $object) {
                    $svc = get_service_by_id($sg->host_id,$sg->service_id);
                    if ($svc !== false) {
                        $objects[] = $svc;
                    }
                }
            }
        }
        break;

        // servicecatalogs
    case "SCAT":
        $host_name = $object;
        $service_description = '';
        $scat = get_scat_by_name($object);
        if ($scat !== false) {
            $objects[] = $scat;
        }
        break;

    }

    return $objects;


}

function usage($only_version = false)
{
    global $valid_periods;

    print "Version: "._VERSION."\n";
    if ($only_version)
        return;

    print "\nUsage:\n";
    print "getavail.php [-v] [-V] [-h] -t <type> -p <period> -o <object> -w <warning> -c <critical> [-p <period>] [-T <timecut name>] [-m] [-s] [-S]\n";
    print "\n";
    print "Options:\n";
    print "-V\n";
    print "\tPrints agent version\n";
    print "-v\n";
    print "\tEnable verbose mode\n";
    print "-h\n";
    print "\tPrints this help message\n";
    print "-t <type>\n";
    print "\tObject type, it should be: IC(Host),AIC(Service),ICG(HostGroup),AICG(ServiceGroup) or SCAT(Service Catalog)\n";
    print "-o <object>\n";
    print "\tObject description, for service use host_name:service_description\n";
    print "-w <warning>\n";
    print "\tWarning threshold(if the availability is below this limit, a warning alert will be generated\n";
    print "-c <critical>\n";
    print "\tCritical threshold(if the availability is below this limit, a critical alert will be generated\n";
    print "-p <period>\n";
    print "\tAvail time period. It must be: ".implode(", ",$valid_periods)."\n";
    print "\tDefault value points to last7days\n";
    print "-T <timecut name>\n";
    print "\tUse this option to specify a timecut different from '24x7'\n";
    print "-m \n";
    print "\tCreate a new metric to present the time in minutes per state\n";
    print "-s \n";
    print "\tConsider scheduled downtimes and breach reason periods as OK\n";
    print "\n";
    print "-S \n";
    print "\tUse soft states\n";
    print "-C \n";
    print "\tUse for count state changes\n";
    print "\n";
    print "Examples:\n";
    print "\tgetavail.php -t IC -o 'hostX' -p thisyear\n";
    print "\t\tCalcule the availability for hostX host this year\n\n";
    print "\tgetavail.php -t AIC -o 'hostX:serviceY' -p last7days -T '8x5'\n";
    print "\t\tCalcule the availability for serviceY service on hostX host for the last 7 days using timecut 8x5\n\n";
    print "\tgetavail.php -t IC -o 'hostX' -p today -w 79 -c 75\n";
    print "\t\tCalcule the availability for hostX host today. Generates critical alert if availability < 75% and\n";
    print "\t\twarning alert if availiability > 75% and < 79%\n";

    print "\n";
}

exit(main());
