package main

import (
    "fmt"
    "os"
    "net/http"
    "crypto/tls"
    "flag"
    "bytes"
    "encoding/json"
)

var (
    username = flag.String("username", "", "OpMon username\n")
    password = flag.String("password", "", "OpMon password\n")
    address = flag.String("address", "", "URL NFCe\n")
    host = flag.String("host", "", "Host Name\n")
    service = flag.String("service", "", "Service Description\n")
    verbose = flag.Bool("verbose", false, "Show verbose")
)

func main() {

    flag.Parse()
 
    required := []string{"username", "password", "address", "host"}
    seen := make(map[string]bool)
    flag.Visit(func(f *flag.Flag) { seen[f.Name] = true })
    for _, req := range required {
        if !seen[req] {
            fmt.Fprintf(os.Stderr, "missing required -%s argument\n", req)
            os.Exit(2)
        }
    }

    tr := &http.Transport{
            TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
    }
    client := http.Client{Transport: tr}

    var jsonprep string = `{"host_name":"`+*host+`","service_description":"`+*service+`"}`

    url := "https://"+*address+"/opmon/seagull/www/index.php/wsconnector/action/"
    endpoint := "host"
    if *service != "" {
        endpoint = "service"
    }
    url = url+endpoint

    var jsonStr = []byte("jsonData="+jsonprep)

    if *verbose == true {
        fmt.Println("URL: "+url)
        fmt.Println("Username: "+*username)
        fmt.Println("Password: "+*password)
        fmt.Println("Data: "+string(jsonprep))
    }

    req, err := http.NewRequest("POST", url, bytes.NewBuffer(jsonStr))
    req.Header.Set("Content-Type", "application/x-www-form-urlencoded")
    req.SetBasicAuth(*username, *password)

    resp, err := client.Do(req)
    if err != nil {
        fmt.Println(err)
        os.Exit(3)
    }

    var data map[string]interface{}
    json.NewDecoder(resp.Body).Decode(&data)
    
    success := data["success"]
    state := int(data["object"].(map[string]interface{})["state"].(float64))
    perf_data := data["object"].(map[string]interface{})["perf_data"]
    plugin_output := data["object"].(map[string]interface{})["plugin_output"]
    
    if success != true {
        fmt.Println("Object not found")
        os.Exit(1)        
    }

    fmt.Print(plugin_output)
    fmt.Print("|")
    fmt.Println(perf_data)
    os.Exit(state)
}
