# Monitor share quota

if ($args.Count -lt 3) {
   Write-Host "Missing parameters, parameters are Path, Warning and Critical"
   Exit(3)
}
$Path = $args[0]
$Warning = $args[1]
$Critical = $args[2]

$QuotaCMD = dirquota.exe q l /Path:$Path /Type:HARD


$QuotaCMD | Select-String -Pattern "Quota Path" -Context 0,9 |
	%{$share = "" | Select-Object Path, Description, SharePath, Template, Status, Limit, LimitUnit, Used, UsedUnit, Available, AvailableUnit;
	$share.Path = $_.Line.Split(":")[1].TrimStart(" ") + ":" + $_.Line.Split(":")[2].TrimStart(" ");
    $share.Description = $_.Context.PostContext[0].Split(":")[1].TrimStart(" "); 
    $share.SharePath = $_.Context.PostContext[1].Split(":")[1].TrimStart(" "); 
	$share.Template = $_.Context.PostContext[2].Split(":")[1].TrimStart(" "); 
	$share.Status = $_.Context.PostContext[3].Split(":")[1].TrimStart(" ");
	$share.Limit = $_.Context.PostContext[4].Split(":")[1].TrimStart(" ").Split(" ")[0];
	$share.LimitUnit = $_.Context.PostContext[4].Split(":")[1].TrimStart(" ").Split(" ")[1];
	$share.Used = $_.Context.PostContext[5].Split(":")[1].TrimStart(" ").Split(" ")[0];
	$share.UsedUnit = $_.Context.PostContext[5].Split(":")[1].TrimStart(" ").Split(" ")[1];
	$share.Available = $_.Context.PostContext[6].Split(":")[1].TrimStart(" ").Split(" ")[0];
	$share.AvailableUnit = $_.Context.PostContext[6].Split(":")[1].TrimStart(" ").Split(" ")[1];
    $share}   |  Foreach {$_.Line} > combined3.txt

if ($share.Path -ne $Path) {
   Write-Host "Patch $Path not found"
   Exit(3)
}

# Convert all metrics do GB
$share.LimitUnit = $share.LimitUnit.ToUpper()
$share.UsedUnit = $share.UsedUnit.ToUpper()

$Size = 0
if ($share.LimitUnit -eq "TB") {
	$Size = [math]::Round($share.Limit * 1024, 2)
}
if ($share.LimitUnit -eq "GB") {
	$Size = [math]::Round($share.Limit, 2)
}
if ($share.LimitUnit -eq "MB") {
	$Size = [math]::Round($share.Limit / 1004, 2)
}
if ($share.LimitUnit -eq "KB") {
	$Size = [math]::Round($share.Limit / 1004, 2)
}

$Used = 0
if ($share.UsedUnit -eq "TB") {
	$Used = [math]::Round($share.Used * 1024, 2)
}
if ($share.UsedUnit -eq "GB") {
	$Used = [math]::Round($share.Used, 2)
}
if ($share.UsedUnit -eq "MB") {
	$Used = [math]::Round($share.Used / 1004, 2)
}
if ($share.UsedUnit -eq "KB") {
	$Used = [math]::Round($share.Used / 1004, 2)
}

$Free = [math]::Round(($Size - $Used), 2)
$Percent = [math]::Round($Used*100/$Size, 2)

$Perfdata = "Free=" + $Free + "GB;;;0;" + $Size + " Size=" + $Size + "GB;;;0;" + $Size + " Used=" + $Used+ "GB;;;0;" + $Size

$msg = "OK"
$state = 0
if ($Percent -gt $Warning) {
	$msg = "Warning"
	$state = 1
}
if ($Percent -gt $Critical) {
	$msg = "Critical"
	$state = 2
}

$plugin_output = $msg + " " + $Free + " GB free of " + $Size + " GB (" + $Percent + " %) | " + $Perfdata

#Write-Host $plugin_output
Write-Output $plugin_output
Exit($state)
