# Monitor share quota

if ($args.Count -lt 3) {
   Write-Host "Missing parameters, parameters are Path, Warning and Critical"
   Exit(3)
}
$Path = $args[0]
$Warning = $args[1]
$Critical = $args[2]

# "D:\BKP\Ponto4"
Try {
	$share = Get-FsrmQuota -Path $Path
}
Catch [CimException] {
   Write-Host "Patch $Path not found"
   Exit(3)
}

if ($share.Path -ne $Path) {
   Write-Host "Patch $Path not found"
   Exit(3)
}

$Used = [math]::Round($share.Usage / 1GB, 2)
$Size = [math]::Round($share.Size / 1GB, 2)
$Free = [math]::Round(($share.Size - $share.Usage)  / 1GB, 2)
$Percent = [math]::Round($share.Usage*100/$share.Size, 2)

$Perfdata = "Free=" + $Free + "GB;;;0;" + $Size + " Size=" + $Size + "GB;;;0;" + $Size + " Used=" + $Used+ "GB;;;0;" + $Size

$msg = "OK"
$state = 0
if ($Percent -gt $Warning) {
	$msg = "Warning"
	$state = 1
}
if ($Percent -gt $Critical) {
	$msg = "Critical"
	$state = 2
}

$plugin_output = $msg + " " + $Free + " GB free of " + $Size + " GB (" + $Percent + " %) | " + $Perfdata

#Write-Host $plugin_output
Write-Output $plugin_output
Exit($state)

