package main

import (
	"fmt"
	"os"
    "strings"

	livestatus "github.com/vbatoufflet/go-livestatus"
)

func main() {
    if (len(os.Args) <= 1) {
        fmt.Printf("Usage: check_ic_status $HOSTNAME$\n")
        os.Exit(2)
    }
	hostname := os.Args[1]

	c := livestatus.NewClient("tcp", "localhost:6557")
	defer c.Close()

	q := livestatus.NewQuery("services")
	q.Columns(
        "description",
        "display_status",
        "state_type",
        "process_performance_data",
    )
	q.Filter("host_name = " + hostname)

	resp, err := c.Exec(q)
	if err != nil {
		fmt.Fprintf(os.Stderr, "Error: %s", err)
		os.Exit(1)
	}

    services := len(resp.Records)
    availProblems := []string{}
    unknowns := 0
	for _, r := range resp.Records {
		svc, err := r.GetString("description")
		if err != nil {
			fmt.Fprintf(os.Stderr, "Warning: %s", err)
		}

		status, err := r.GetInt("display_status")
		if err != nil {
			fmt.Fprintf(os.Stderr, "Warning: %s", err)
		}

		stateType, err := r.GetInt("state_type")
		if err != nil {
			fmt.Fprintf(os.Stderr, "Warning: %s", err)
		}

		perfData, err := r.GetInt("process_performance_data")
		if err != nil {
			fmt.Fprintf(os.Stderr, "Warning: %s", err)
		}

        if (stateType == 1) {
            if (perfData == 0 && (status == 5 || status == 6)) {
                availProblems = append(availProblems, svc)
            }

            if (status == 5) {
                unknowns++
            }
        }
    }

    if (services == 0) {
		fmt.Printf("IC sem KPIs\n")
        os.Exit(2)
    }

    if (services == unknowns) {
		fmt.Printf("IC - Desconhecido (Verificar problemas de checagem)\n")
        os.Exit(2)
    }

    if (len(availProblems) > 0) {
		fmt.Printf("IC - Critical: %s\n", strings.Join(availProblems, ", "))
        os.Exit(2)
    }

    fmt.Printf("IC - Ok\n")
    os.Exit(0)
}
